"""
Tests for the Google Analytics template tags and filters, using the new gtag.js library.
"""

from django.contrib.auth.models import User
from django.http import HttpRequest
from django.template import Context
from django.test.utils import override_settings

from analytical.templatetags.google_analytics_gtag import GoogleAnalyticsGTagNode
from utils import TagTestCase
from analytical.utils import AnalyticalException


@override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='UA-123456-7')
class GoogleAnalyticsTagTestCase(TagTestCase):
    """
    Tests for the ``google_analytics_js`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('google_analytics_gtag', 'google_analytics_gtag')
        self.assertTrue(
            '<script async src="https://www.googletagmanager.com/gtag/js?id=UA-123456-7"></script>'
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'UA-123456-7');" in r, r)

    def test_node(self):
        r = GoogleAnalyticsGTagNode().render(Context())
        self.assertTrue(
            '<script async src="https://www.googletagmanager.com/gtag/js?id=UA-123456-7"></script>'
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'UA-123456-7');" in r, r)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID=None)
    def test_no_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsGTagNode)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='wrong')
    def test_wrong_property_id(self):
        self.assertRaises(AnalyticalException, GoogleAnalyticsGTagNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = GoogleAnalyticsGTagNode().render(context)
        self.assertTrue(r.startswith(
            '<!-- Google Analytics disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = GoogleAnalyticsGTagNode().render(Context({'user': User(username='test')}))
        self.assertTrue("gtag('set', {'user_id': 'test'});" in r, r)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='G-12345678')
    def test_tag_with_measurement_id(self):
        r = self.render_tag('google_analytics_gtag', 'google_analytics_gtag')
        self.assertTrue(
            ('<script async src="https://www.googletagmanager.com/gtag/' +
             'js?id=G-12345678"></script>')
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'G-12345678');" in r, r)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='AW-1234567890')
    def test_tag_with_conversion_id(self):
        r = self.render_tag('google_analytics_gtag', 'google_analytics_gtag')
        self.assertTrue(
            ('<script async src="https://www.googletagmanager.com/gtag/' +
             'js?id=AW-1234567890"></script>')
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'AW-1234567890');" in r, r)

    @override_settings(GOOGLE_ANALYTICS_GTAG_PROPERTY_ID='DC-12345678')
    def test_tag_with_advertiser_id(self):
        r = self.render_tag('google_analytics_gtag', 'google_analytics_gtag')
        self.assertTrue(
            ('<script async src="https://www.googletagmanager.com/gtag/' +
             'js?id=DC-12345678"></script>')
            in r, r)
        self.assertTrue("gtag('js', new Date());" in r, r)
        self.assertTrue("gtag('config', 'DC-12345678');" in r, r)



"""
Tests for the Lucky Orange template tags.
"""
from django.http import HttpRequest
from django.template import Context, Template, TemplateSyntaxError
from django.test import override_settings

from analytical.templatetags.analytical import _load_template_nodes
from analytical.templatetags.luckyorange import LuckyOrangeNode
from utils import TagTestCase
from analytical.utils import AnalyticalException


expected_html = """\
<script type='text/javascript'>
window.__lo_site_id = 123456;
(function() {
    var wa = document.createElement('script'); wa.type = 'text/javascript'; wa.async = true;
    wa.src = 'https://d10lpsik1i8c69.cloudfront.net/w.js';
    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(wa, s);
})();
</script>
"""


@override_settings(LUCKYORANGE_SITE_ID='123456')
class LuckyOrangeTagTestCase(TagTestCase):

    maxDiff = None

    def test_tag(self):
        html = self.render_tag('luckyorange', 'luckyorange')
        self.assertEqual(expected_html, html)

    def test_node(self):
        html = LuckyOrangeNode().render(Context({}))
        self.assertEqual(expected_html, html)

    def test_tags_take_no_args(self):
        self.assertRaisesRegex(
            TemplateSyntaxError,
            r"^'luckyorange' takes no arguments$",
            lambda: (Template('{% load luckyorange %}{% luckyorange "arg" %}')
                     .render(Context({}))),
        )

    @override_settings(LUCKYORANGE_SITE_ID=None)
    def test_no_id(self):
        expected_pattern = r'^LUCKYORANGE_SITE_ID setting is not set$'
        self.assertRaisesRegex(AnalyticalException, expected_pattern, LuckyOrangeNode)

    @override_settings(LUCKYORANGE_SITE_ID='invalid')
    def test_invalid_id(self):
        expected_pattern = (
            r"^LUCKYORANGE_SITE_ID setting: must be \(a string containing\) a number: 'invalid'$")
        self.assertRaisesRegex(AnalyticalException, expected_pattern, LuckyOrangeNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        request = HttpRequest()
        request.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': request})

        actual_html = LuckyOrangeNode().render(context)
        disabled_html = '\n'.join([
                '<!-- Lucky Orange disabled on internal IP address',
                expected_html,
                '-->',
            ])
        self.assertEqual(disabled_html, actual_html)

    def test_contribute_to_analytical(self):
        """
        `luckyorange.contribute_to_analytical` registers the head and body nodes.
        """
        template_nodes = _load_template_nodes()
        self.assertEqual({
            'head_top': [],
            'head_bottom': [LuckyOrangeNode],
            'body_top': [],
            'body_bottom': [],
        }, template_nodes)



"""
Matomo template tags and filters.
"""

from collections import namedtuple
from itertools import chain
import re

from django.conf import settings
from django.template import Library, Node, TemplateSyntaxError

from analytical.utils import (is_internal_ip, disable_html,
                              get_required_setting, get_identity)


# domain name (characters separated by a dot), optional port, optional URI path, no slash
DOMAINPATH_RE = re.compile(r'^(([^./?#@:]+\.)*[^./?#@:]+)+(:[0-9]+)?(/[^/?#@:]+)*$')

# numeric ID
SITEID_RE = re.compile(r'^\d+$')

TRACKING_CODE = """
<script type="text/javascript">
  var _paq = window._paq || [];
  %(variables)s
  %(commands)s
  _paq.push(['trackPageView']);
  _paq.push(['enableLinkTracking']);
  (function() {
    var u="//%(url)s/";
    _paq.push(['setTrackerUrl', u+'matomo.php']);
    _paq.push(['setSiteId', %(siteid)s]);
    var d=document, g=d.createElement('script'), s=d.getElementsByTagName('script')[0];
    g.type='text/javascript'; g.async=true; g.defer=true; g.src=u+'matomo.js'; s.parentNode.insertBefore(g,s);
  })();
</script>
<noscript><p><img src="//%(url)s/piwik.php?idsite=%(siteid)s" style="border:0;" alt="" /></p></noscript>
"""  # noqa

VARIABLE_CODE = '_paq.push(["setCustomVariable", %(index)s, "%(name)s", "%(value)s", "%(scope)s"]);'  # noqa
IDENTITY_CODE = '_paq.push(["setUserId", "%(userid)s"]);'
DISABLE_COOKIES_CODE = '_paq.push([\'disableCookies\']);'

DEFAULT_SCOPE = 'page'

MatomoVar = namedtuple('MatomoVar', ('index', 'name', 'value', 'scope'))


register = Library()


@register.tag
def matomo(parser, token):
    """
    Matomo tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your Matomo domain (plus optional URI path), and tracked site ID
    in the ``MATOMO_DOMAIN_PATH`` and the ``MATOMO_SITE_ID`` setting.

    Custom variables can be passed in the ``matomo_vars`` context
    variable.  It is an iterable of custom variables as tuples like:
    ``(index, name, value[, scope])`` where scope may be ``'page'``
    (default) or ``'visit'``.  Index should be an integer and the
    other parameters should be strings.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return MatomoNode()


class MatomoNode(Node):
    def __init__(self):
        self.domain_path = \
            get_required_setting('MATOMO_DOMAIN_PATH', DOMAINPATH_RE,
                                 "must be a domain name, optionally followed "
                                 "by an URI path, no trailing slash (e.g. "
                                 "matomo.example.com or my.matomo.server/path)")
        self.site_id = \
            get_required_setting('MATOMO_SITE_ID', SITEID_RE,
                                 "must be a (string containing a) number")

    def render(self, context):
        custom_variables = context.get('matomo_vars', ())

        complete_variables = (var if len(var) >= 4 else var + (DEFAULT_SCOPE,)
                              for var in custom_variables)

        variables_code = (VARIABLE_CODE % MatomoVar(*var)._asdict()
                          for var in complete_variables)

        commands = []
        if getattr(settings, 'MATOMO_DISABLE_COOKIES', False):
            commands.append(DISABLE_COOKIES_CODE)

        userid = get_identity(context, 'matomo')
        if userid is not None:
            variables_code = chain(variables_code, (
                IDENTITY_CODE % {'userid': userid},
            ))

        html = TRACKING_CODE % {
            'url': self.domain_path,
            'siteid': self.site_id,
            'variables': '\n  '.join(variables_code),
            'commands': '\n  '.join(commands)
        }
        if is_internal_ip(context, 'MATOMO'):
            html = disable_html(html, 'Matomo')
        return html


def contribute_to_analytical(add_node):
    MatomoNode()  # ensure properly configured
    add_node('body_bottom', MatomoNode)



import os

from setuptools import setup

import analytical as package


def read_file(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='django-analytical',
    version=package.__version__,
    license=package.__license__,
    description=package.__doc__.strip(),
    long_description=read_file('README.rst'),
    long_description_content_type='text/x-rst',
    author=package.__author__,
    author_email=package.__email__,
    packages=[
        'analytical',
        'analytical.templatetags',
    ],
    keywords=[
        'django',
        'analytics',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3.6',
    platforms=['any'],
    url='https://github.com/jazzband/django-analytical',
    download_url='https://github.com/jazzband/django-analytical/archive/master.zip',
    project_urls={
        'Documentation': 'https://django-analytical.readthedocs.io/',
    },
)
